import random

# function that lists all the available functions in the module
### the parser help is used instead ###
    
# ---------------------------------------------------------------------------- #
#                              available functions                             #
# ---------------------------------------------------------------------------- #


def hello(name):
    print('\U0001F438: Hi there ' + name + '!')


def encourage():
    frogs_decision = random.randint(1, 3)
    
    if frogs_decision == 1:
        print('\U0001F438: You are the smartest frog in the pond!')
    elif frogs_decision == 2:
        print('\U0001F438: You got this! Leap by leap!')
    else:
        print('\U0001F438: You are amazing!')
    

def frogmode():
    print("\U0001F438: I EAT BUGS, let's eat bugs together")


def feed(snack):
    cap_snack = snack.capitalize()
    print("\U0001F438: Thanks for the sweet little treat! " + cap_snack + " is yummy! nom nom nom")
    
    
def kill(anything):
    frogs_decision = random.randint(1, 2)
    
    # frog is willing to kill for you!
    if frogs_decision == 1:
        print("\U0001F438: I killed " + anything + " for you! You are safe now!")
    
    # nuarr the frog is a pacifist
    else:
        print("\U0001F438: I don't like " + anything + ", I only eat yummy snacks! nom nom nom")
    
    
def goodbye(name):
    print('\U0001F438: Bye bye ' + name+ ', go catch some flies for me!')
    





import pytest
from ribbet.ribbet import hello, encourage, frogmode, feed, kill, goodbye
# hello
def test_hello(capsys):
    hello("Alice")
    captured = capsys.readouterr()
    assert "\U0001F438: Hi there Alice!" in captured.out

# encourage  
def test_encourage(capsys):
    encourage()
    captured = capsys.readouterr()
    assert "\U0001F438:" in captured.out

@pytest.mark.parametrize("seed", [1, 2, 3])
def test_encourage_variants(capsys, monkeypatch, seed):
    def mock_randint(a, b):
        return seed
    monkeypatch.setattr("random.randint", mock_randint)
    encourage()
    captured = capsys.readouterr()
    if seed == 1:
        assert "\U0001F438: You are the smartest frog in the pond!" in captured.out
    elif seed == 2:
        assert "\U0001F438: You got this! Leap by leap!" in captured.out
    else:  # seed == 3
        assert "\U0001F438: You are amazing!" in captured.out

# frogmode     
def test_frogmode(capsys):
    frogmode()
    captured = capsys.readouterr()
    assert "\U0001F438: I EAT BUGS, let's eat bugs together" in captured.out

# feed  
def test_feed(capsys):
    feed("flies")
    captured = capsys.readouterr()
    assert "\U0001F438: Thanks for the sweet little treat! Flies is yummy! nom nom nom" in captured.out

# kill  
def test_kill(capsys):
    kill("the bad vibes")
    captured = capsys.readouterr()
    assert "\U0001F438:" in captured.out

@pytest.mark.parametrize("seed,expected", [
    (1, "I killed the bad vibes for you! You are safe now!"),
    (2, "I don't like the bad vibes, I only eat yummy snacks! nom nom nom")
])
def test_kill_variants(capsys, monkeypatch, seed, expected):
    def mock_randint(a, b):
        return seed
    monkeypatch.setattr("random.randint", mock_randint)
    kill("the bad vibes")
    captured = capsys.readouterr()
    assert "\U0001F438: " + expected in captured.out

# goodbye
def test_goodbye(capsys):
    goodbye("Alice")
    captured = capsys.readouterr()
    assert "\U0001F438: Bye bye Alice, go catch some flies for me!" in captured.out


"""
# help
def test_help(capsys):
    help()
    captured = capsys.readouterr()
    assert "hello(your name): the frog says hello to you ;)" in captured.out
    assert "encourage(): the frog gives you a silly little pep talk <3" in captured.out
    assert "frogmode(): the frogs gets into frog mode and eats bugs with you!" in captured.out
    assert "feed(snack): feed the frog a yummy snack!" in captured.out
    assert "kill(anything): the fighter frog kills anything for you!" in captured.out
    assert "goodbye(your name): frog says goodbye to you!" in captured.out
"""






import argparse
import sys
from ribbet import ribbet

class Custom_ArgParser(argparse.ArgumentParser):
    def error(self, message):
        # stderr -> standard error in linux
        sys.stderr.write(f'ERROR: {message}\n')
        sys.stderr.write("Please enter the correct amount of arguments.\n")
        print()
        # prints help table to show commands & args 
        # this line is key in overriding generic error handler 
        self.print_help()
        sys.exit(2)
        
def args_parser():
    parser = Custom_ArgParser(description="Welcome to the frog module!")
    
    # the function wanted to be called
    subparsers = parser.add_subparsers(help="command", dest="command")

    
    hello_parser = subparsers.add_parser("hello", help="the frog says hello to you")
    hello_parser.add_argument("name", help="your name")
    
    subparsers.add_parser("encourage", help="the frog gives you a silly little pep talk <3")
    
    subparsers.add_parser("frogmode", help="the frogs gets into frog mode and eats bugs with you!")
    
    feed_parser = subparsers.add_parser("feed", help="feed the frog a yummy snack!")
    feed_parser.add_argument("snack", help="the snack you would like to feed the frog")
    
    kill_parser = subparsers.add_parser("kill", help="the fighter frog kills anything for you!")    
    kill_parser.add_argument("anything", help="anything you want the frog to kill for you")
    
    goodbye_parser = subparsers.add_parser("goodbye", help="frog says goodbye to you!")
    goodbye_parser.add_argument("name", help="your name")
    
    return parser

""" 
# this function doesnt work - needs to override this with custom class 
def handle_argument_error(e):
    print(f"Error: {str(e)}")
    print("Please enter the correct amount of arguments")
"""

def main():
    parser = args_parser()
    
    try:
        args = parser.parse_args()
        # call respective function
        if args.command == "hello":
            ribbet.hello(args.name)
        elif args.command == "encourage":
            ribbet.encourage()
        elif args.command == "frogmode":
            ribbet.frogmode()
        elif args.command == "feed":
            ribbet.feed(args.snack)
        elif args.command == "kill":
            ribbet.kill(args.anything)
        elif args.command == "goodbye":
            ribbet.goodbye(args.name)
    except Exception as e:  # Catching generic exception for demonstration; customize as needed
        #handle_argument_error()
        print(e)  # Optionally print the error message

if __name__ == "__main__":
    main()





import pytest
from unittest.mock import patch
from ribbet import ribbet_cli

@pytest.mark.parametrize("test_args,expected_call", [
    (['prog', 'frogmode'], 'frogmode'),
    (['prog', 'feed', 'flies'], ('feed', 'flies')),
    (['prog', 'hello', 'Alice'], ('hello', 'Alice')),
])

def test_frog_cli_commands(test_args, expected_call):
    with patch('sys.argv', test_args), patch(f'ribbet.ribbet.{expected_call[0] if isinstance(expected_call, tuple) else expected_call}', return_value=None) as mock_func:
        ribbet_cli.main()
        if isinstance(expected_call, tuple):
            mock_func.assert_called_once_with(*expected_call[1:])
        else:
            mock_func.assert_called_once()

def test_frogmode_no_arguments():
    test_args = ['prog', 'frogmode']
    with patch('sys.argv', test_args), patch('ribbet.ribbet.frogmode', return_value=None) as mock_func:
        ribbet_cli.main()
        mock_func.assert_called_once_with()

def test_hello_with_numbers():
    test_args = ['prog', 'hello', '123']
    with patch('sys.argv', test_args), patch('ribbet.ribbet.hello', return_value=None) as mock_func:
        ribbet_cli.main()
        mock_func.assert_called_once_with('123')

def test_hello_with_special_characters():
    test_args = ['prog', 'hello', '!@#$%']
    with patch('sys.argv', test_args), patch('ribbet.ribbet.hello', return_value=None) as mock_func:
        ribbet_cli.main()
        mock_func.assert_called_once_with('!@#$%')

